@@ -262,7 +262,7 @@ class Makefile:
         if dir:
             self.dir = os.path.abspath(dir)
         else:
-            self.dir = os.path.curdir
+            self.dir = os.getcwd()
 
         # Assume we are building in the source tree.
         self._src_dir = self.dir
@@ -319,6 +319,7 @@ class Makefile:
         self.extra_lflags = []
         self.extra_lib_dirs = []
         self.extra_libs = []
+        self.extra_source_dirs = []
 
         # Get these once and make them available to sub-classes.
         if sys.platform == "win32":
@@ -1199,8 +1200,13 @@ class Makefile:
         if self._qt:
             mfile.write("MOC = %s\n" % _quote(self.required_string("MOC")))
 
+        vpath = _UniqueList(self.extra_source_dirs)
+
         if self._src_dir != self.dir:
-            mfile.write("VPATH = %s\n\n" % self._src_dir)
+            vpath.append(self._src_dir)
+
+        if vpath.as_list():
+            mfile.write("VPATH = %s\n\n" % " ".join(vpath.as_list()))
 
         # These probably don't matter.
         if self.generator == "MINGW":
@@ -1554,16 +1560,6 @@ class ModuleMakefile(Makefile):
         self.LFLAGS.extend(self.optional_list(lflags_console))
 
         if sys.platform == "darwin":
-            from distutils.sysconfig import get_python_inc
-
-            # The Python include directory seems to be the only one that uses
-            # the real path even when using a virtual environment (eg. pyvenv).
-            # Note that I can't remember why we need a framework build.
-            dl = get_python_inc().split(os.sep)
-
-            if "Python.framework" not in dl:
-                error("SIP requires Python to be built as a framework")
-
             self.LFLAGS.append("-undefined dynamic_lookup")
 
         Makefile.finalise(self)
@@ -1687,7 +1683,7 @@ class ModuleMakefile(Makefile):
             cpp = "moc_" + root + ".cpp"
 
             mfile.write("\n%s: %s\n" % (cpp, mf))
-            mfile.write("\t$(MOC) -o %s %s\n" % (cpp, mf))
+            mfile.write("\t$(MOC) -o %s $<\n" % cpp)
 
         mfile.write("\n$(TARGET): $(OFILES)\n")
 
@@ -1984,7 +1980,7 @@ class ProgramMakefile(Makefile):
                 mf = os.path.join(self._src_dir, mf)
 
             mfile.write("\n%s: %s\n" % (cpp, mf))
-            mfile.write("\t$(MOC) -o %s %s\n" % (cpp, mf))
+            mfile.write("\t$(MOC) -o %s $<\n" % cpp)
 
         mfile.write("\n$(TARGET): $(OFILES)\n")
 
@@ -2040,11 +2036,20 @@ def _quote(s):
     return s
 
 
-def version_to_string(v):
-    """Convert a 3 part version number encoded as a hexadecimal value to a
-    string.
+def version_to_string(version, parts=3):
+    """ Convert an n-part version number encoded as a hexadecimal value to a
+    string.  version is the version number.  Returns the string.
     """
-    return "%u.%u.%u" % (((v >> 16) & 0xff), ((v >> 8) & 0xff), (v & 0xff))
+
+    part_list = [str((version >> 16) & 0xff)]
+
+    if parts > 1:
+        part_list.append(str((version >> 8) & 0xff))
+
+        if parts > 2:
+            part_list.append(str(version & 0xff))
+
+    return '.'.join(part_list)
 
 
 def read_version(filename, description, numdefine=None, strdefine=None):
@@ -2455,10 +2460,10 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
             else:
                 break
 
-        line = line.strip()
+        # Strip comments and surrounding whitespace.
+        line = line.split('#', 1)[0].strip()
 
-        # Ignore comments.
-        if line and line[0] != "#":
+        if line:
             assstart = line.find("+")
             if assstart > 0 and line[assstart + 1] == '=':
                 adding = True
@@ -2637,7 +2642,8 @@ def create_wrapper(script, wrapper, gui=0, use_arch=''):
     script is the full pathname of the script.
     wrapper is the name of the wrapper file to create.
     gui is non-zero if a GUI enabled version of the interpreter should be used.
-    use_arch is the MacOS/X architecture to invoke python with.
+    use_arch is the MacOS/X architectures to invoke python with.  Several space
+    separated architectures may be specified.
 
     Returns the platform specific name of the wrapper.
     """
@@ -2657,18 +2663,26 @@ def create_wrapper(script, wrapper, gui=0, use_arch=''):
         # The installation of MacOS's python is a mess that changes from
         # version to version and where sys.executable is useless.
 
+        version = sys.version_info
+        py_major = version[0]
+        py_minor = version[1]
+
         if gui:
-            exe = "pythonw"
+            # In Python v3.4 and later there is no pythonw.
+            if (py_major == 3 and py_minor >= 4) or py_major >= 4:
+                exe = "python"
+            else:
+                exe = "pythonw"
         else:
             exe = "python"
 
-        version = sys.version_info
-        exe = "%s%d.%d" % (exe, version[0], version[1])
+        exe = "%s%d.%d" % (exe, py_major, py_minor)
 
         if use_arch:
             # Note that this may not work with the "standard" interpreter but
             # should with the "pythonX.Y" version.
-            exe = "arch -%s %s" % (use_arch, exe)
+            arch_flags = ' '.join(["-%s" % a for a in use_arch.split()])
+            exe = "arch %s %s" % (arch_flags, exe)
 
         wf.write("#!/bin/sh\n")
         wf.write("exec %s %s ${1+\"$@\"}\n" % (exe, script))
